//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// CentralInstall.as
//
// Utilities to help install apps in Central
//
// Copyright 2002 Macromedia
//
//--------------------------------------
// resolveUrl(url)
//--------------------------------------
//
// Resolves a relative url against _root._url.
//
// If you are playing
//		'http://www.domain.com/folder/subfolder/movie.swf'
//
// then resolveUrl('service.swf') returns
//		'http://www.domain.com/folder/subfolder/service.swf'
//
// and resolveUrl('../service.swf') returns
//		'http://www.domain.com/folder/service.swf'
//
// and resolveUrl('../../service.swf') returns
//		'http://www.domain.com/service.swf'
//
// and resolveUrl('../../../service.swf') returns
//		'http://service.swf'
//
// (aka: garbage. Use it in a way you know will work)
//
//
//--------------------------------------
// fcInstallService(respObj, url, count)
//--------------------------------------
//
// You can use this class by
//	myIS = new fcInstallService(respObj, url, count);
//	myIS.start();
//
// There are no other methods to call. If Central
// is not installed, it will download it. If it is no
// running, it will launch it. Once running, it will
// use a LocalConnection to send() the installService()
// request to Central.
//
//
// url must be absolute ... see function resolveUrl() for relative url's
//
// Central connect()'s it's LocalConnections after it's first frame.
// If we launch the application, we do not know how long this will take,
// so we will re-try the send() once per second for up to count
// seconds if we launched the application ... if it was already running,
// we will try only once.
//
// respObj.onStatus() may be invoked by start(), so be sure your code
// can handle this (my original code couldn't).
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// resolveUrl(url)
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
function resolveUrl(url)
{
	// start with 'http://www.domain.com/folder/subfolder/movie.swf'
	var base = _root._url;

	// strip last component 'http://www.domain.com/folder/subfolder'
	base = base.slice(0, base.lastIndexOf('/'));

	// does url start with "../" (../service.swf)
	while (url.slice(0, 3) == "../")
	{
		// strip last component, 'again http://www.domain.com/folder'
		base = base.slice(0, base.lastIndexOf('/'));

		// strip leading "../" (service.swf)
		url = url.slice(3);
	}

	// return url ('http://www.domain.com/folder' + '/' + 'service.swf'
	return base + '/' + url;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// fcisStatusConnection is a subclass of LocalConnection.
//
//	{level:"status", code:"InstallProduct.Started"}
//	{level:"status", code:"InstallProduct.Success"}
//	{level:"error",  code:"InstallProduct.Error"}
//	{level:"error",  code:"InstallProduct.Failed"}
//	{level:"error",  code:"InstallProduct.Cancelled"}
//
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

function fcisStatusConnection(  respObj, connectionString )
{
	// get rid of the prefix on the url
	this.connectionString =  connectionString;
	this.respObj = respObj;

	this.connect( this.connectionString );
}

fcisStatusConnection.prototype = new LocalConnection();

fcisStatusConnection.prototype.allowDomain = function(domain)
{
	//myTrace("allowDomain " + domain);
	if (domain == "macromedia.com")
       	return true;
}

fcisStatusConnection.prototype.InstallStatus = function(info)
{
	if ( info.code == "InstallProduct.Success" ||  info.code == "InstallProduct.Failed"  ||
		info.code == "InstallProduct.Cancelled" ||  info.code == "InstallProduct.Error")
		this.close();

	this.respObj.lcStatus( info );
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// fcisLocalConnection is a subclass of LocalConnection.
//
// fcInstallService will create an instance with itself as the respObj.
//
// It will only call doInstall(count), which tries to send() an
// installService message to Central. If count is non-zero,
// and the message fails, then fcisLocalConnection will resend
// the message once per second for up to count seconds.
//
// respObj.onStatus() will be called with :
//	{level:"status", code:"InstallProduct.LocalConnection.SendPending"}
//	{level:"status", code:"InstallProduct.LocalConnection.SendSuccess"}
//	{level:"error",  code:"InstallProduct.LocalConnection.SendFailed"}
//
// respectively each time fcisLocalConnection send()'s, when
// fcisLocalConnection is done because the send() was dispatched, and
// when fcisLocalConnection is done because we've failed for more than
// count seconds.
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

function fcisLocalConnection(respObj, url, statusConnection)
{
	this.url		= url;
	this.respObj	= respObj;
	this.statusConnection = this.domain() + ":" + statusConnection;
}

fcisLocalConnection.prototype = new LocalConnection();

fcisLocalConnection.prototype.finish = function(theLevel, theCode)
{
	// Stop the interval (if it's started), and send the final status
	if (this.intervalID)
	{
		clearInterval(this.intervalID);
		delete this.intervalID;
	}

	this.respObj.lcStatus( {level : theLevel, code : theCode} );
}

fcisLocalConnection.prototype.onStatus = function(info)
{
	// onStatus() is called in response to a send().

	if (info.level == "status")
	{
		// The message was dispatched.
		this.finish("status", "InstallProduct.LocalConnection.SendSuccess");
	}
	else
	{
		// The message was dropped.
		// Assume Central hasn't connected, but will

		if ((this.count > 0) && !this.intervalID)
		{
			// This is the first time we failed.
			//
			// Start a one second interval, and retry until count hits zero.
			this.intervalID = setInterval(fcisPollDoInstall, 1000, this);
		}
		else if (this.count < 1)
		{
			// We've retried for as long as we were supposed to.
			this.finish("error", "InstallProduct.LocalConnection.SendFailed");
		}
	}
};

fcisLocalConnection.prototype.doInstall = function(count)
{
	this.count = count; // number of seconds to retry, if the first send() fails.

	this.send("macromedia.com:central", "installService", this.url, this.statusConnection);
	this.respObj.lcStatus( {level : "status", code : "InstallProduct.LocalConnection.SendPending"} );
};


fcisPollDoInstall = function(lc)
{
	lc.doInstall(lc.count - 1);
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// fcisCentral is a subclass of System.product.
//
// fcInstallService will create an instance with itself as the respObj.
//
// Other code shouldn't invoke the methods of fcisCentral, as it's
// constructor will take the steps necessary to make sure that
// Central is installed and running.
//
// respObj.onStatus() will be called with :
//	{level:"status", code:"Central.Running"}
//	{level:"status", code:"Central.Download.Pending"}
//	{level:"error",  code:"Central.Download.Failed"}
//	{level:"status", code:"Central.Launch.Success"}
//	{level:"error",  code:"Central.Launch.Failed"}
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

fcisCentral = function(respObj)
{
	this.respObj = respObj;

	if (this.isRunning())
	{
		this.respObj.fcStatus( {level : "status", code : "Central.Running"} );
	}
	else if (this.isInstalled())
	{
		if (this.launch())
			this.respObj.fcStatus( {level : "status", code : "Central.Launch.Success"} );
		else
			this.respObj.fcStatus( {level : "error", code : "Central.Launch.Failed"} );
	}
	else
	{
		if (this.download())
			this.respObj.fcStatus( {level : "status", code : "Central.Download.Pending"} );
		else
			this.respObj.fcStatus( {level : "error", code : "Central.Download.Failed"} );
	}
}

fcisCentral.prototype = new System.Product("Central");

fcisCentral.prototype.onStatus = function(info)
{
	// onStatus() is invoked when the download() stops.

	if (info.code == "Download.Complete")
	{
		// The download worked. Launch the application.

		if (this.launch())
			this.respObj.fcStatus( {level : "status", code : "Central.Launch.Success"} );
		else
			this.respObj.fcStatus( {level : "error", code : "Central.Launch.Failed"} );
	}
	else
	{
		this.respObj.fcStatus( {level : "error", code : "Central.Download.Failed"} );
	}
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// fcInstallService is a subclass of Object.
//
// It will potentially download and/or launch Central before
// sending an installService request to a known LocalConnection made
// by Central.
//
// The url passed to it must be absolute.
//
// If Central had to be launched, the send may be tried for
// up to count seconds, as we wait for the other process to connect().
//
// respObj.onStatus() will be invoked with
//	{level:"status", code:"Central.Download.Pending"}
//	{level:"error",  code:"Central.Download.Failed"}
//	{level:"status", code:"Central.Launch.Success"}
//	{level:"error",  code:"Central.Launch.Failed"}
//	{level:"status", code:"Central.Running"}
//
//	{level:"status", code:"InstallProduct.LocalConnection.SendPending"}
//	{level:"status", code:"InstallProduct.LocalConnection.SendSuccess"}
//	{level:"error",  code:"InstallProduct.LocalConnection.SendFailed"}
//
//	{level:"status", code:"InstallProduct.Started"}
//	{level:"status", code:"InstallProduct.Success"}
//	{level:"error",  code:"InstallProduct.Error"}
//	{level:"error",  code:"InstallProduct.Failed"}
//	{level:"error",  code:"InstallProduct.Cancelled"}
//
//
// fcInstallService is done when it code "InstallService.Success" or
// "InstallService.Failed".
//
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

function fcInstallService(respObj, url, count)
{
	this.respObj	= respObj;
	this.count		= count;
	this.url		= url;

	// constructing fc will cause it to download and/or launch
	// Central and invoke callback this.fcStatus when complete.
}

fcInstallService.prototype = new Object();

fcInstallService.prototype.start = function()
{
	// in case start's been called previously, reset our state.
	delete this.lc;
	delete this.fc;
	delete this.sc;

	// Originally, this was in fcInstallService(). However, it
	// took a half day of debugging to realise that some error
	// codes happen synchronously ... so the respObj.onStatus()
	// which was meant to clear a global variable when we were
	// done was being called before we assigned to the global
	// variable. So, I broke this into peices so that respObj
	// is never invoked during construction of fcInstallService.

	if (this.url != "")
	{
		var connectionString  = this.url.slice(this.url.indexOf(':')+3, this.url.length) + 	"/InstallResponse";

		this.sc			= new fcisStatusConnection( this, connectionString);
		this.lc			= new fcisLocalConnection(this, this.url, connectionString );
	}
	this.fc			= new fcisCentral(this);
}

fcInstallService.prototype.lcStatus = function(info)
{
	// Expect one of :
	//	{level:"status", code:"InstallProduct.LocalConnection.SendPending"}
	//	{level:"status", code:"InstallProduct.LocalConnection.SendSuccess"}
	//	{level:"error",  code:"InstallProduct.LocalConnection.SendFailed"}
	//
	//	{level:"status", code:"InstallProduct.Started"}
	//	{level:"status", code:"InstallProduct.Success"}
	//	{level:"error",  code:"InstallProduct.Error"}
	//	{level:"error",  code:"InstallProduct.Failed"}
	//	{level:"error",  code:"InstallProduct.Cancelled"}
	//
	// forward the message
	this.respObj.onStatus(info);
}

fcInstallService.prototype.fcStatus = function(info)
{
	// Expect one of :
	//	{level:"status", code:"Central.Running"}
	//	{level:"status", code:"Central.Launch.Success"}
	//	{level:"status", code:"Central.Download.Pending"}
	//	{level:"error",  code:"Central.Download.Failed"}
	//	{level:"error",  code:"Central.Launch.Failed"}

	// forward the message
	this.respObj.onStatus(info);

	if (this.url != "")
	{
		if (info.level == "status")
		{
			if (info.code == "Central.Running")
			{
				// Central is already running, send the message once.
				this.lc.doInstall(0);
			}
			else if (info.code == "Central.Launch.Success")
			{
				// Central was just launched. send the message, but
				// if it fails, retry once a second for up to 20 seconds.
				this.lc.doInstall(this.count);
			}
		}
		else
		{
			// generate our own status for terminal states.
			this.respObj.onStatus({level:"error",  code:"InstallProduct.Failed"});
		}
	}
}
